/* Copyright (C) 2003 Damir Zucic */

/*=============================================================================

			    hyphob_function3.c

Purpose:
	Draw the hydrophobicity function F3.  The sequence stored to the
	main sequence buffer  is used  to calculate  the function value.

Input:
	(1) Pointer to the storage where the minimal function value will
	    be stored.
	(2) Pointer to the storage where the maximal function value will
	    be stored.
	(3) Pointer to RuntimeS structure.

Output:
	(1) Function F3 calculated and stored.
	(2) Return value.

Return value:
	(1) Positive on success.
	(2) Negative on failure.

Notes:
	(1) The function  F3 may be modified and used for many purposes.
	    Originally, it was introduced while searching for the method
	    which will be suitable for prediction of the porin secondary
	    structure.

========includes:============================================================*/

#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "defines.h"
#include "typedefs.h"

/*======calculate the hydrophobicity function F3:============================*/

int HyphobFunction3_ (double *min_functionP, double *max_functionP,
		      RuntimeS *runtimeSP)
{
int             residuesN, residueI;   /* Do not use size_t instead of int ! */
int		max_length;
int		windowI, combinedI;
int		used_residuesN;
char		*residue_nameP;
char		residue_nameA[RESNAMESIZE];
int		residue_from_setF;
double		function_value;

/*------prepare some parameters:---------------------------------------------*/

/* The number of residues in sequence buffer: */
residuesN = (int) runtimeSP->residuesN;
if (residuesN == 0) return -1;

/* The maximal residue name length: */
max_length = RESNAMESIZE - 1;

/*------calculate the function F3:-------------------------------------------*/

/* Initialize the extreme values: */
*min_functionP = +999999.0;
*max_functionP = -999999.0;

/* 20040104.1517: */
/* A window of 9 residues is scanned. The number of GLY, ALA, SER, */
/* THR and ASN  on the same side is stored as  the function value. */

/* Scan the sequence: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Reset the counts of residues used to prepare the raw values: */
	used_residuesN = 0;

	/* Reset the function value, it might be initialized before: */
	*(runtimeSP->function3P + residueI) = 0.0;

	/* Reset the local variable reserved for the function value: */
	function_value = 0.0;

	/* Scan the sliding window: */
	for (windowI = 0; windowI < 9; windowI += 2)
		{
		/* Prepare and check the combined index: */
		combinedI = residueI + windowI - 4;
		if (combinedI < 0) continue;
		if (combinedI >= residuesN) continue;

		/* Pointer  to  the residue  name of */
		/* the residue defined by combinedI: */
		residue_nameP = runtimeSP->sequenceP + combinedI * max_length;

		/* Copy the residue name: */
		strncpy (residue_nameA, residue_nameP, max_length);
		residue_nameA[max_length] = '\0';

		/* Now check the residue name. */

		/* Initial assumption: residue belongs to the specified set: */
		residue_from_setF = 1;

		/* Formal, single pass loop: */
		do
			{
			/* Check the residue name: */
			if (strcmp (residue_nameA, "GLY") == 0) break;
			else if (strcmp (residue_nameA, "ALA") == 0) break;
			else if (strcmp (residue_nameA, "SER") == 0) break;
			else if (strcmp (residue_nameA, "THR") == 0) break;
			else if (strcmp (residue_nameA, "ASN") == 0) break;

			/* If this point is reached, this residue */
			/* does not belong  to the specified set. */
			residue_from_setF = 0;

			/* End of single pass loop: */
			} while (0);

		/* If this residue belongs to the given */
		/* set,  add one to the function value: */
		if (residue_from_setF == 1) function_value += 1.0;

		/* Update the number of used residues: */
		used_residuesN++;
		}

	/* Reduce the function value (arbitrary scaling): */
	function_value *= 0.1;

	/* Store the function value: */
	*(runtimeSP->function3P + residueI) = function_value;

        /* Dummy values should not be used to determine extreme values: */
        if (used_residuesN < 1) continue;

	/* Find the extreme values: */
	if (function_value < *min_functionP) *min_functionP = function_value;
	if (function_value > *max_functionP) *max_functionP = function_value;

	/* End of residueI loop: */
	}

/*---------------------------------------------------------------------------*/

return 1;
}

/*===========================================================================*/


