/* Copyright (C) 2003 Damir Zucic */

/*=============================================================================

			    hyphob_function4.c

Purpose:
	Draw the hydrophobicity function F4.  The sequence stored to the
	main sequence buffer  is used  to calculate  the function value.

Input:
	(1) Pointer to the storage where the minimal function value will
	    be stored.
	(2) Pointer to the storage where the maximal function value will
	    be stored.
	(3) Pointer to RuntimeS structure.

Output:
	(1) Function F4 calculated and stored.
	(2) Return value.

Return value:
	(1) Positive on success.
	(2) Negative on failure.

Notes:
	(1) The function  F4 may be modified and used for many purposes.
	    Originally, it was introduced while searching for the method
	    which will be suitable for prediction of the porin secondary
	    structure.

========includes:============================================================*/

#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "defines.h"
#include "typedefs.h"

/*======calculate the hydrophobicity function F4:============================*/

int HyphobFunction4_ (double *min_functionP, double *max_functionP,
		      RuntimeS *runtimeSP)
{
int             residuesN, residueI;   /* Do not use size_t instead of int ! */
int		windowI, combinedI;
int		used_residuesN;
double		function_value;
double		hydrophobicity;

static double	weightA[13] = {-1.0, -1.0, -1.0, -1.0, -1.0, -1.0,
				0.0,
				1.0,  1.0,  1.0,  1.0,  1.0,  1.0};

/*------prepare some parameters:---------------------------------------------*/

/* The number of residues in sequence buffer: */
residuesN = (int) runtimeSP->residuesN;
if (residuesN == 0) return -1;

/*------calculate the function F4:-------------------------------------------*/

/* Initialize the extreme values: */
*min_functionP = +999999.0;
*max_functionP = -999999.0;

/* 20031204.2045: */
/* A window of  13 residues is scanned.  The weighted hydrophobicity is */
/* calculated and used as the function value. This function is based on */
/* hydrophobic hexapeptide model  (W. C. Wimley et al., S. White lab.). */

/* Initialize (reset) the function value: */
for (residueI = 0; residueI < residuesN; residueI++)
	{
	/* Reset the function value, it might be initialized before: */
	*(runtimeSP->function4P + residueI) = 0.0;
	}

/* Scan the sequence, ignoring six residues at each end: */
for (residueI = 6; residueI < residuesN - 6; residueI++)
	{
	/* Reset the number of residues used to prepare the raw values: */
	used_residuesN = 0;

	/* Reset the local variable which is used for the function value: */
	function_value = 0.0;

	/* Scan the sliding window: */
	for (windowI = 0; windowI < 13; windowI++)
		{
		/* Prepare and check the combined index: */
		combinedI = residueI + windowI - 6;
		if (combinedI < 0) continue;
		if (combinedI >= residuesN) continue;

		/* Prepare the hydrophobicity value: */
		hydrophobicity = *(runtimeSP->hydrophobicityP + combinedI);

		/* Add the weighted  hydrophobicity of */
		/* this residue to the function value: */
		function_value += weightA[windowI] * hydrophobicity;

		/* Update the number of used residues: */
		used_residuesN++;
		}

	/* Check the number of used residues: */
	if (used_residuesN == 0) continue;


	/* Scale the function value (it should fit */
	/* to the same plot with other functions): */
	function_value /= 12.0;

	/* Store the function value: */
	*(runtimeSP->function4P + residueI) = function_value;

	/* Dummy values should not be used to determine extreme values: */
	if (used_residuesN < 13) continue;

	/* Find the extreme values: */
	if (function_value < *min_functionP) *min_functionP = function_value;
	if (function_value > *max_functionP) *max_functionP = function_value;

	/* End of residueI loop: */
	}

/*---------------------------------------------------------------------------*/

return 1;
}

/*===========================================================================*/


